<?php

if ( ! defined( 'ABSPATH' ) ) {
  exit;
}

if ( ! class_exists( 'MywpModel' ) ) :

final class MywpModel {

  private $model_id = false;
  private $controller_id = false;
  private $model_type = false;
  private $network = false;

  private $initial_data = false;
  private $default_data = false;

  private $option_key = false;
  private $option_autoload = false;

  private $pre_get_option = false;
  private $option = false;
  private $after_get_option = false;

  private $pre_setting_data = false;
  private $setting_data = false;
  private $after_setting_data = false;

  public function __construct( $controller_id = false , $model_type = false , $network = false ) {

    if( empty( $controller_id ) ) {

      $called_text = sprintf( 'new %s( %s , %s , %s )' , __CLASS__ , '$controller_id' , '$model_type' , '$network' );

      MywpHelper::error_require_message( '$controller_id' , $called_text );

      return false;

    }

    $this->controller_id = strip_tags( $controller_id );

    if( empty( $model_type ) ) {

      $called_text = sprintf( 'new %s( %s , %s , %s )' , __CLASS__ , '$controller_id' , '$model_type' , '$network' );

      MywpHelper::error_require_message( '$model_type' , $called_text );

      return false;

    }

    $this->model_type = strip_tags( $model_type );

    $this->model_id = "mywp_{$this->controller_id}";

    if( ! empty( $network ) ) {

      $this->network = true;

    }

  }

  public function get_model_id() {

    return $this->model_id;

  }

  public function get_model_type() {

    return $this->model_type;

  }

  public function is_network() {

    return $this->network;

  }

  public function set_initial_data( $initial_data = false ) {

    $this->initial_data = $initial_data;

  }

  public function get_initial_data() {

    return $this->initial_data;

  }

  public function set_default_data( $default_data = false ) {

    $this->default_data = $default_data;

  }

  public function get_default_data() {

    return $this->default_data;

  }

  public function get_option_key() {

    $model_id = $this->get_model_id();

    if( empty( $model_id ) ) {

      return false;

    }

    $this->option_key = apply_filters( "mywp_get_option_key_{$model_id}" , $model_id , $this->model_type , $this->network );

    return $this->option_key;

  }

  public function get_option_autoload() {

    $model_id = $this->get_model_id();

    if( empty( $model_id ) ) {

      return false;

    }

    $this->option_autoload = apply_filters( "mywp_get_option_autoload_{$model_id}" , $this->option_autoload , $this->model_type , $this->network );

    return $this->option_autoload;

  }

  public function get_data() {

    $option_key = $this->get_option_key();

    if( empty( $option_key ) ) {

      $called_text = sprintf( '(object) %s->%s()' , __CLASS__ , __FUNCTION__ );

      MywpHelper::error_not_found_message( '$option_key' , $called_text );

      return false;

    }

    do_action( 'mywp_pre_get_data_option' , $option_key , $this->model_type , $this->network );

    $this->pre_get_option = apply_filters( "mywp_pre_get_data_{$option_key}" , false , $this->model_type , $this->network );

    if( $this->pre_get_option !== false ) {

      return $this->pre_get_option;

    }

    if( $this->network ) {

      $this->option = get_site_option( $option_key );

    } else {

      $this->option = get_option( $option_key );

    }

    $this->after_get_option = apply_filters( "mywp_after_get_data_{$option_key}" , $this->option , $this->model_type , $this->network );

    do_action( 'mywp_after_get_data_option' , $option_key , $this->model_type , $this->network );

    if( $this->after_get_option !== false ) {

      return $this->after_get_option;

    }

    return $this->option;

  }

  public function update_data( $update_data ) {

    $option_key = $this->get_option_key();

    if( empty( $option_key ) ) {

      $called_text = sprintf( '(object) %s->%s()' , __CLASS__ , __FUNCTION__ );

      MywpHelper::error_not_found_message( '$option_key' , $called_text );

      return false;

    }

    if( $this->network ) {

      $return = update_site_option( $option_key , $update_data );

    } else {

      $option_autoload = $this->get_option_autoload();

      $return = update_option( $option_key , $update_data , $option_autoload );

    }

    do_action( "mywp_after_update_data_{$option_key}" , $update_data , $return , $this->model_type , $this->network );

    return $return;

  }

  public function remove_data() {

    $option_key = $this->get_option_key();

    if( empty( $option_key ) ) {

      $called_text = sprintf( '(object) %s->%s()' , __CLASS__ , __FUNCTION__ );

      MywpHelper::error_not_found_message( '$option_key' , $called_text );

      return false;

    }

    if( $this->network ) {

      $return = delete_site_option( $option_key );

    } else {

      $return = delete_option( $option_key );

    }

    do_action( "mywp_after_remove_data_{$option_key}" , $return , $this->model_type , $this->network );

    return $return;

  }

  public function get_setting_data() {

    $option_key = $this->get_option_key();

    if( empty( $option_key ) ) {

      $called_text = sprintf( '(object) %s->%s()' , __CLASS__ , __FUNCTION__ );

      MywpHelper::error_not_found_message( '$option_key' , $called_text );

      return false;

    }

    $this->pre_setting_data = apply_filters( "mywp_pre_get_setting_data_{$option_key}" , false , $this->model_type , $this->network );

    if( $this->pre_setting_data !== false ) {

      $this->pre_setting_data = wp_parse_args( $this->pre_setting_data , $this->get_initial_data() );

      return $this->pre_setting_data;

    }

    $this->setting_data = $this->get_data();

    if( $this->setting_data === false ) {

      $this->setting_data = $this->get_default_data();

    }

    $this->setting_data = wp_parse_args( $this->setting_data , $this->get_initial_data() );

    $this->after_setting_data = apply_filters( "mywp_after_get_setting_data_{$option_key}" , $this->setting_data , $this->model_type , $this->network );

    if( $this->after_setting_data !== false ) {

      return $this->after_setting_data;

    }

    return $this->setting_data;

  }

}

endif;
